<?
/*  SMF: NNTP Syndicate Mod
 *
 *	Based on:
 *	Newsportal NNTP<->HTTP Gateway
 *  Version: 0.36
 *  Download: http://florian-amrhein.de/newsportal
 *
 *  Copyright (C) 2002-2004 Florian Amrhein
 *  E-Mail: newsportal@florian-amrhein.de
 *  Web: http://florian-amrhein.de
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

if (!defined('SMF'))
	die('Hacking attempt...');

/* -----------------------------------------------------------------------
 * Post Functions
 * ----------------------------------------------------------------------- */

/*
 * Convert and remove HTML tags.
 *
 * $body: the body to be cleaned up.
 *
 * the function returns a sting containing the cleaned up body.
 */
function html_cleanup($body) {
	// Convert <br> types to \r\n
	$new_body = str_replace(array("<br>","<br/>","<br />"), array("\r\n","\r\n","\r\n"), $body);
	
	// Convert &nbsp; to space
	$new_body = str_replace("&nbsp;", " ", $new_body);

	// Strip all HTML tags
	$new_body = strip_tags($new_body);
	
	return $new_body;
}

/*
 * Integrate post subjects.
 *
 * $nntp_subject: the subject template for the NNTP post.
 * $ubject: the subject of the actual forum post.
 *
 * the function returns a sting containing the actual nntp post suject.
 */
function integrate_subjects($nntp_subject, $subject) {
	$new_subject = str_replace("[SUBJECT]", $subject, $nntp_subject);
	
	return $new_subject;
}

/*
 * Integrate post bodies and insert link.
 *
 * $nntp_body: the body template for the NNTP post.
 * $body: the body of the actual forum post.
 * $boardurl: the URL of the forum.
 * $topic_id: the topic_id of the forum post.
 *
 * the function returns a sting containing the actual nntp post body.
 */
function integrate_bodies($nntp_body, $subject, $body, $topic_id) {
	global $boardurl;
	
	$new_body = str_replace("[SUBJECT]", $subject, $nntp_body);
	$new_body = str_replace("[BODY]", $body, $new_body);
	$new_body = str_replace("[LINK]", $boardurl."/index.php?topic=".$topic_id.".0", $new_body);
	
	return $new_body;
}

/*
 * Wrap text
 *
 */
function textwrap($text, $wrap=80, $break="\n",$maxlen=false){
  $len = strlen($text);
  if ($len > $wrap) {
    $h = '';        // massaged text
    $lastWhite = 0; // position of last whitespace char
    $lastChar = 0;  // position of last char
    $lastBreak = 0; // position of last break
    // while there is text to process
    while ($lastChar < $len && (($maxlen==false) || (strlen($h)<$maxlen))) {
      $char = substr($text, $lastChar, 1); // get the next character
      // if we are beyond the wrap boundry and there is a place to break
      if (($lastChar - $lastBreak > $wrap) && ($lastWhite > $lastBreak)) {
        $h .= substr($text, $lastBreak, ($lastWhite - $lastBreak)) . $break;
        $lastChar = $lastWhite + 1;
        $lastBreak = $lastChar;
      }
      // You may wish to include other characters as valid whitespace...
      if ($char == ' ' || $char == chr(13) || $char == chr(10)) {
        $lastWhite = $lastChar; // note the position of the last whitespace
      }
      $lastChar = $lastChar + 1; // advance the last character position by one
    }
    $h .= substr($text, $lastBreak); // build line
  } else {
    $h = $text; // in this case everything can fit on one line
  }
  return $h;
}

/*
 * Encode lines with 8bit-characters to quote-printable
 *
 * $line: the to be encoded line
 *
 * the function returns a sting containing the quoted-printable encoded
 * $line
 */
function quoted_printable_encode($line) {
  global $www_charset;
  $qp_table=array(
     '=00', '=01', '=02', '=03', '=04', '=05',
     '=06', '=07', '=08', '=09', '=0A', '=0B',
     '=0C', '=0D', '=0E', '=0F', '=10', '=11',
     '=12', '=13', '=14', '=15', '=16', '=17',
     '=18', '=19', '=1A', '=1B', '=1C', '=1D',
     '=1E', '=1F', '_',   '!',   '"',   '#',
     '$',   '%',   '&',   "'",   '(',   ')',
     '*',   '+',   ',',   '-',   '.',   '/',
     '0',   '1',   '2',   '3',   '4',   '5',
     '6',   '7',   '8',   '9',   ':',   ';',
     '<',   '=3D', '>',   '=3F', '@',   'A',
     'B',   'C',   'D',   'E',   'F',   'G',
     'H',   'I',   'J',   'K',   'L',   'M',
     'N',   'O',   'P',   'Q',   'R',   'S',
     'T',   'U',   'V',   'W',   'X',   'Y',
     'Z',   '[',   '\\',  ']',   '^',   '=5F',
     '',    'a',   'b',   'c',   'd',   'e',
     'f',   'g',   'h',   'i',   'j',   'k',
     'l',   'm',   'n',   'o',   'p',   'q',
     'r',   's',   't',   'u',   'v',   'w',
     'x',   'y',   'z',   '{',   '|',   '}',
     '~',   '=7F', '=80', '=81', '=82', '=83',
     '=84', '=85', '=86', '=87', '=88', '=89',
     '=8A', '=8B', '=8C', '=8D', '=8E', '=8F',
     '=90', '=91', '=92', '=93', '=94', '=95',
     '=96', '=97', '=98', '=99', '=9A', '=9B',
     '=9C', '=9D', '=9E', '=9F', '=A0', '=A1',
     '=A2', '=A3', '=A4', '=A5', '=A6', '=A7',
     '=A8', '=A9', '=AA', '=AB', '=AC', '=AD',
     '=AE', '=AF', '=B0', '=B1', '=B2', '=B3',
     '=B4', '=B5', '=B6', '=B7', '=B8', '=B9',
     '=BA', '=BB', '=BC', '=BD', '=BE', '=BF',
     '=C0', '=C1', '=C2', '=C3', '=C4', '=C5',
     '=C6', '=C7', '=C8', '=C9', '=CA', '=CB',
     '=CC', '=CD', '=CE', '=CF', '=D0', '=D1',
     '=D2', '=D3', '=D4', '=D5', '=D6', '=D7',
     '=D8', '=D9', '=DA', '=DB', '=DC', '=DD',
     '=DE', '=DF', '=E0', '=E1', '=E2', '=E3',
     '=E4', '=E5', '=E6', '=E7', '=E8', '=E9',
     '=EA', '=EB', '=EC', '=ED', '=EE', '=EF',
     '=F0', '=F1', '=F2', '=F3', '=F4', '=F5',
     '=F6', '=F7', '=F8', '=F9', '=FA', '=FB',
     '=FC', '=FD', '=FE', '=FF');
  // are there "forbidden" characters in the string?
  for($i=0; $i<strlen($line) && ord($line[$i])<=127 ; $i++);
  if ($i<strlen($line)) { // yes, there are. So lets encode them!
    $from=$i;
    for($to=strlen($line)-1; ord($line[$to])<=127; $to--);
    // lets scan for the start and the end of the to be encoded _words_
    for(;$from>0 && $line[$from] != ' '; $from--);
    if($from>0) $from++;
    for(;$to<strlen($line) && $line[$to] != ' '; $to++);
    // split the string into the to be encoded middle and the rest
    $begin=substr($line,0,$from);
    $middle=substr($line,$from,$to-$from);
    $end=substr($line,$to);
    // ok, now lets encode $middle...
    $newmiddle="";
    for($i=0; $i<strlen($middle); $i++)
      $newmiddle .= $qp_table[ord($middle[$i])];
    // now we glue the parts together...
    $line=$begin.'=?'.$www_charset.'?Q?'.$newmiddle.'?='.$end;
  }
  return $line;
}

/*
 * generate a message-id for posting.
 * $identity: a string containing informations about the article, to
 *     make a md5-hash out of it.
 *
 * returns: a complete message-id
 */
function generate_msgid($identity) {
	// Variables
	$msgid_generate="md5";
	$msgid_fqdn=$_SERVER["HTTP_HOST"];

  switch($msgid_generate) {
    case "no":
      // no, we don't want to generate a message-id.
      return false;
      break;
    case "md5":
      return '<'.md5($identity).'$1@'.$msgid_fqdn.'>';
      break;
    default:
      return false;
      break;
  }
}

/*
 * Post an article to a newsgroup
 *
 * $subject: The Subject of the article
 * $from: The authors name and email of the article
 * $newsgroups: The groups to post to
 * $ref: The references of the article
 * $body: The article itself
 */
function nntp_message_post($subject,$name,$email,$newsgroups,$ref,$body,$nntp_subject,$nntp_body,$topic_id) {
	global $modSettings,$boardurl,$txt,$sourcedir;

	// Variables
	$send_poster_host=false;
	$www_charset="iso-8859-15";
	$file_footer="";
	$user_agent="SMF Mod: NNTP Syndicate (http://custom.simplemachines.org/mods/)";

  flush();
  $ns=nntp_open($modSettings['nntp_syndicate_server'],$modSettings['nntp_syndicate_port']);
  if ($ns != false) {
    fputs($ns,"POST\r\n");
    $weg=line_read($ns);

    // Integrate subject of forum post into actual newsgroup post
    $new_subject = integrate_subjects($nntp_subject, $subject);
    
    fputs($ns,"Subject: ".quoted_printable_encode($new_subject)."\r\n");
    fputs($ns,"From: ".$email." (".quoted_printable_encode($name).")\r\n");
    fputs($ns,"Newsgroups: ".$newsgroups."\r\n");
    fputs($ns,"Mime-Version: 1.0\r\n");
    fputs($ns,"Content-Type: text/plain; charset=".$www_charset."; format=flowed\r\n");
    fputs($ns,"Content-Transfer-Encoding: 8bit\r\n");
    fputs($ns,"User-Agent: ".$user_agent."\r\n");
    if ($send_poster_host)
      @fputs($ns,'X-HTTP-Posting-Host: '.gethostbyaddr(getenv("REMOTE_ADDR"))."\r\n");
    if (($ref!=false) && (count($ref)>0)) {
      // strip references
      if(strlen(implode(" ",$ref))>900) {
        $ref_first=array_shift($ref);
        do {
          $ref=array_slice($ref,1);
        } while(strlen(implode(" ",$ref))>800);
        array_unshift($ref,$ref_first);
      }
      fputs($ns,'References: '.implode(" ",$ref)."\r\n");
    }
    if (isset($boardurl))
      fputs($ns,'Organization: '.quoted_printable_encode($boardurl)."\r\n");
    if ((isset($file_footer)) && ($file_footer!="")) {
      $footerfile=fopen($file_footer,"r");
      $body.="\n".fread($footerfile,filesize($file_footer));
      fclose($footerfile);
    }
    if($msgid=generate_msgid($subject.",".$name.",".$email.",".$newsgroups.",".$ref.",".$body.",".time()))
      fputs($ns,'Message-ID: '.$msgid."\r\n");

    $body=str_replace("\n.\r","\n..\r",$body);
    $body=str_replace("\r",'',$body);
    $b=split("\n",$body);
    $body="";
    for ($i=0; $i<count($b); $i++) {
      if ((strpos(substr($b[$i],0,strpos($b[$i]," ")),">") != false) || (strcmp(substr($b[$i],0,1),">") == 0)) {
        $body .= textwrap(stripSlashes($b[$i]),78," \r\n");
      } else {
        $body .= textwrap(stripSlashes($b[$i]),74," \r\n");
      }
    }
    
    // Parse BBC code
    // include($sourcedir . "/Subs.php");
    $body = parse_bbc($body);
    
    // Convert and remove HTML tags from post
    $body = html_cleanup($body);
    
    // Integrate body of forum post and link into actual newsgroup post
    $body = integrate_bodies($nntp_body, $subject, $body, $topic_id);
    
    // Ensure body contains data, else put some in.
    if (empty($body))
    	$body = " ";
    
    fputs($ns,"\r\n".$body."\r\n.\r\n");
    $message=line_read($ns);
    
    if (substr($message,0,3) != "240") {
    	log_error($txt['nntp_post_failed'] . "<br />" . $message);
    }
    
    nntp_close($ns);
  } else {
    $message=$txt["nntp_connection_failed"];
    log_error($message);
  }

  return $message;
}


/* -----------------------------------------------------------------------
 * NNTP Functions
 * ----------------------------------------------------------------------- */

/*
 * opens the connection to the NNTP-Server
 *
 * $server: adress of the NNTP-Server
 * $port: port of the server
 */
function nntp_open($nserver,$nport) {
  global $modSettings,$boardurl,$txt;

	//Variables
	$server="news.aioe.org";
	$port=119;

  // echo "<br>NNTP OPEN<br>";
  $authorize=!empty($modSettings['nntp_syndicate_authentication']);
  if (empty($nserver)) $nserver=$server;
  if (empty($nport)) $nport=$port;
  $ns=@fsockopen($nserver,$nport,$errno,$errstr);
  $weg=line_read($ns);  // kill the first line
  if (substr($weg,0,2) != "20") {
    log_error($txt["nntp_error"].$weg."<br /><br />$errstr ($errno)");
    fclose($ns);
    $ns=false;
  } else {
    if ($ns != false) {
      fputs($ns,"MODE reader\r\n");
      $weg=line_read($ns);  // and once more
      if ((substr($weg,0,2) != "20") && 
          ((!$authorize) || ((substr($weg,0,3) != "480") && ($authorize)))) {
        log_error($txt["nntp_error"].$weg);
        fclose($ns);
        $ns=false;
      }
    }
    if ($authorize) {
		  $server_auth_user=$modSettings['nntp_syndicate_username'];
  		$server_auth_pass=$modSettings['nntp_syndicate_password'];
      
      fputs($ns,"AUTHINFO USER $server_auth_user\r\n");
      $weg=line_read($ns);
      fputs($ns,"AUTHINFO PASS $server_auth_pass\r\n"); 
      $weg=line_read($ns);
      if (substr($weg,0,3) != "281") {
        log_error($txt["nntp_authentication_error"]);
      }
    }
  }

  return $ns;
}

/*
 * Close a NNTP connection
 *
 * $ns: the handle of the connection
 */
function nntp_close(&$ns) {
  if ($ns != false) {
    fputs($ns,"QUIT\r\n");
    fclose($ns);
  }
}

/*
 * read one line from the NNTP-server
 */
function line_read(&$ns) {
  if ($ns != false) {
    $t=str_replace("\n","",str_replace("\r","",fgets($ns,1200)));
    return $t;
  }
}
?>